import querystring from 'query-string';
import cuid from 'cuid';
import openPopup from './util/popup';

var listenForCredentials = function listenForCredentials(popup, state, resolve, reject) {
  var hash = void 0;
  try {
    hash = popup.location.hash;
  } catch (err) {
    if (process.env.NODE_ENV !== 'production') {
      /* eslint-disable no-console */
      console.error(err);
      /* eslint-enable no-console */
    }
  }

  if (hash) {
    popup.close();

    var response = querystring.parse(hash.substr(1));
    if (response.state !== state) {
      reject('Invalid state returned.');
    }

    if (response.access_token) {
      var expiresIn = response.expires_in ? parseInt(response.expires_in, 10) : NaN;
      var result = {
        token: response.access_token,
        expiresAt: !isNaN(expiresIn) ? Date.now() + expiresIn * 1000 : null
      };
      resolve(result);
    } else {
      reject(response.error || 'Unknown error.');
    }
  } else if (popup.closed) {
    reject('Authentication was cancelled.');
  } else {
    setTimeout(function () {
      return listenForCredentials(popup, state, resolve, reject);
    }, 100);
  }
};

var authorize = function authorize(config) {
  var state = cuid();
  var query = querystring.stringify({
    state: state,
    response_type: 'token',
    client_id: config.client,
    scope: config.scope,
    redirect_uri: config.redirect
  });
  var url = config.url + (config.url.indexOf('?') === -1 ? '?' : '&') + query;
  var width = config.width || 400;
  var height = config.height || 400;
  var popup = openPopup(url, 'oauth2', width, height);

  return new Promise(function (resolve, reject) {
    return listenForCredentials(popup, state, resolve, reject);
  });
};

export default authorize;

export var unauthorize = function unauthorize() {

  return new Promise(function (resolve, reject) {
    try {
      var result = {};
      resolve(result);
    } catch (err) {
      reject('Invalid unauthorize returned.');
    }
  });
};